function fitswrite(data,filename,bitpix,bzero,bscale, headers)

%FITSWRITE saves a Matlab matrix to a FITS file.
%
% FITSWRITE(A,FILENAME) saves A to FILENAME in IEEE double precision format.
%
% FITSWRITE(A,FILENAME,BITPIX) saves A to FILENAME in the format specified
% by BITPIX. Valid values of BITPIX are: -64 (IEEE double), -32 (IEEE
% float), 8 (char or uint8), 16 (short), or 32 (long).
%
% FITSWRITE(A,FILENAME,BITPIX,BZERO,BSCALE) adds the keywords 
% BSCALE and BZERO to the header. Notice that FITSWRITE does 
% NOT scale the entries in A. This should be done prior to calling 
% FITSWRITE, e.g., by the statement A = (A - BZERO)./BSCALE.
% If BSCALE==[] or BZERO==[] then these keywords are ignored.
%
% FITSWRITE(A,FILENAME,BITPIX,BZERO,BSCALE,HEADERS) adds the k 80-character
% cards given in the k-by-80 character array HEADERS to the FITS header in
% FILENAME. It is the user's responsibility that the rows of HEADERS contain
% header cards that conform to the FITS standard.
% 

% R.M. Larsen, SOI-MDI, March 2001, Stanford University.
%
% modified by a. birch  Feb, 2004 to be more complient with
% the FITS standard.  zero pad the data to get an integer number
% of records.  changed the make_card.
%  
% modified by a. birch, May 2012. for input data of more than one
% dimension, permute the first two dimensions.  This is for
% compatability with matlab fitsread
%
%

%Reference:
%                NASA/Science Office of Standards and Technology
%           "Definition of the Flexible Image Transport System (FITS)"
%                        NOST 100-1.0    June 19, 1993

if nargin<2
  error('Not enough input arguments.');
end

%
% May 2012, Aaron Birch
% add a little bit of code to permute the first two dimensions.
% don't do anything for 1d input data
%
nd=ndims(data); %% number of dimensions in the data array
if nd==2
	%
	% if data is 2d array, permute dimensions
	%
	data=permute(data,[2 1]);
elseif nd>=3
	%		
	% if data is 3d or higher, permute only first two dimensions
	%
	data=permute(data,[2 1 3:nd]);
end



if nargin>2
  switch(bitpix)
   case {-64, '-64'}
    format = 'float64';
    byte_pix=8;
   case {-32, '-32'}
    format = 'float';
    byte_pix=4;
   case {8, '8'}
    format = 'uint8';
    byte_pix=1;
   case {16, '16'}
    format = 'short';
    byte_pix=2;
   case {32, '32'}
    format = 'long';
    byte_pix=4;
   otherwise,
    error('Data type specified by BITPIX keyword is not -64, -32, 8, 16, or 32');
  end  
else
  bitpix = -64;
  format = 'float64';
  byte_pix=8;
end

% Analyze dimension of array.
dims=size(data);
naxis = length(dims);
if naxis>999 
  error('Too many dimensions. NAXIS can be at most 999.')
end

% Build FITS header
header_record = [make_card('SIMPLE','T');       
                make_card('BITPIX',bitpix);       
	        make_card('NAXIS',naxis)];
n_card=3;
for i=1:naxis
  header_record = [header_record; ...
		   make_card(['NAXIS',num2str(i)],dims(i))];
end
n_card= n_card+naxis;
if nargin>3 & ~(isempty(bzero) | isempty(bscale))
  header_record = [header_record; ...
		   make_card('BSCALE',bscale); 
		   make_card('BZERO',bzero) ];
  n_card= n_card+2;
end
header_record = [header_record; ...
		   make_card('CREATOR','Matlab'); 
		   make_card('DATE',date)];
n_card = n_card+2;
if nargin>5
  if ~ischar(headers) | size(headers,2) ~= 80
    error('Header cards must be 80 characters')
  end
  header_record = [header_record; ...
		   headers];
  n_card = n_card + size(headers,1);
end
header_record = [header_record; ...
		   make_card('END','')];
n_card = n_card+1;
n_blanks = 36 - (rem(n_card-1,36)+1);
header_record = [header_record; ...
		 char(32)*ones(n_blanks,80)];

fid=fopen(filename,'w','ieee-be');
fwrite(fid,header_record','uchar');

fwrite(fid,data,format);

%
% pad with zeros so we have an integer number of records
%
%
L=length(data(:));
BYTES_PER_RECORD=2880;
N=BYTES_PER_RECORD/byte_pix; %% number of data points per record
m=N-rem(L,N); %% pad out to next integer multiple of N
if (m>0 & m<N)
  fwrite(fid,zeros(1,m),format);
end;


fclose(fid);




function c=make_card(keyword,value,varargin)
% Make Keyword card for FITS header
  
  
c = char(32*ones(1,80));
  
if nargin>3
  error('make_card can only use two or three input arguments')
end
  
%
% fits standard requires uppercase
% for keywords
%
keyword=upper(keyword);

  
if ~length(value) %% empty value
  s=sprintf('%s',keyword);
  len=min(length(s),80);
  c(1:len) = s(1:len);
    
elseif strcmp(value,'T') | strcmp(value,'F') %% for logicals
  s=sprintf('%s',keyword);
  len=min(length(s),80);
  c(1:len) = s(1:len);
  c(30)=value;
  c(9)='=';
  
elseif isstr(value) %% for strings
  s=sprintf('%s',keyword);
  len=min(length(s),80);
  c(1:len) = s(1:len);
  
  if length(value)<8 %% short string
    c(12:11+length(value))=value;
    c(11)=char(39);
    c(20)=char(39);
  elseif length(value)>68 %% long string
    value=value(1:68);
    c(12:11+length(value))=value;
    c(11)=char(39);
    c(12+length(value))=char(39);
  else
    c(12:11+length(value))=value;
    c(11)=char(39);
    c(12+length(value))=char(39);
  end
  
  c(9)='=';
    

elseif isreal(value) %% real number
  s=sprintf('%s',keyword);
  c(1:min(length(s),80)) = s(1:min(length(s),80));
  
  if value==round(value) % integer
    s=sprintf('%d',value);
  else
    %
    % i changed this from %f to %e
    % %f can lead to loss of digits
    % ACB
    % also need to change the lower case e
    % to uppercase
    s=upper(sprintf('%e',value)); % float
  end
    
  c(31-length(s):30)=s;
  c(9)='=';
else
  error('cant figure out the type of value, probably complex #');
end

%
% this next bit of code deal with adding the optional comment
% string
% ACB Mon Aug 13 16:46:11 CEST 2012
%
if nargin==3 
  %
  % comment string is optional third argument
  %
  comment=[' / ' strtrim(varargin{1})];
  
  %
  % find last character that is not a space
  %
  last=find(c~=' ',1,'last');
  %
  % put the comment into a string
  % and also trim trailing white space
  %
  %
  s=sprintf('%s',comment);
  %
  % max allowable length of the comment
  %
  % if last is 80, the the allowed length is zero
  %
  max_allowed_len=80-last;
  
  %
  % first three characters are ' / ' 
  % so don't bother unless we have room for
  % at least four characters
  %
  if max_allowed_len>=4
    %
    % trim the comment down to the
    % maximum allowed length
    % and i[
    %
    if length(s)>max_allowed_len
      s=s(1:max_allowed_len);
    end
    %
    % length of the comment
    %
    len=length(s);
    
    %
    % now fill up c
    %
    index_1=last+1;
    index_2=min(last+len,80);
    c(index_1:index_2)=s;
  end

end %%% end if comment string
  
 
