function [v,u,omega,k,t,model]=load_eigs3(p)

% load_eigs3 replaced load_eigs2 on 31 March 2011
%
% Main Changes include
%
% - allowing different types of eigenfunctions to be read in

% Load eigensystem solutions for system similar to that given on
% p608 in Birch et al 2004, but for no damping. Each eigenvalue solution 
% consists of an eigenfunction pair (y1,y2) and corresponding  eigenvalue 
% pair (omega,k). The k grid is chosen a priori and consequentely this is 
% the same for all modes; but clearly (for each k) the omega, y1, y2 triplet
% clearly changes.

% eig_file: base name for the eigenfunction files
% MAX_N:    maximum number of modes to use
% SKIP:     thins out the eigenfunctions in k (redundant when = 1)
name  = p.eig_file;
MIN_N = p.min_n;
MAX_N = p.max_n;
SKIP  = 1;

a=my_load([name '0'],'first eigenfunction file');
if strcmp(p.eig_type, 'vel_y1y2') || strcmp(p.eig_type, 'vel_y1u')
    [Nk,Nt]=size(a.y1(1:SKIP:end,:));
elseif strcmp(p.eig_type, 'disp_w1w2') || strcmp(p.eig_type, 'disp_w1xiz')
    [Nk,Nt]=size(a.w1(1:SKIP:end,:));
end
efunc1 = zeros(MAX_N+1,Nk,Nt);
efunc2 = zeros(MAX_N+1,Nk,Nt);
omega  = zeros(MAX_N+1,Nk);

for i = MIN_N:MAX_N
    try
        a=my_load([name num2str(i)],'eigenfunction file');
    catch ME
        disp(['ERROR: load_eigs3.m : failed to find eigenfunction file ' name num2str(i)]);
        rethrow(ME);
    end
    if strcmp(p.eig_type,'vel_y1y2')
        efunc1(i+1,:,:) = reshape(a.y1(1:SKIP:end,:),[1 Nk Nt]);
        efunc2(i+1,:,:) = reshape(a.y2(1:SKIP:end,:),[1 Nk Nt]);
    elseif strcmp(p.eig_type,'vel_y1u')
        efunc1(i+1,:,:) = reshape(a.y1(1:SKIP:end,:),[1 Nk Nt]);
        efunc2(i+1,:,:) = reshape(a.u(1:SKIP:end,:),[1 Nk Nt]);
    elseif strcmp(p.eig_type,'disp_w1w2')
        efunc1(i+1,:,:) = reshape(a.w1(1:SKIP:end,:),[1 Nk Nt]);
        efunc2(i+1,:,:) = reshape(a.w2(1:SKIP:end,:),[1 Nk Nt]);
    elseif strcmp(p.eig_type,'disp_w1xiz')
        efunc1(i+1,:,:) = reshape(a.w1(1:SKIP:end,:),[1 Nk Nt]);
        efunc2(i+1,:,:) = reshape(a.xiz(1:SKIP:end,:),[1 Nk Nt]);
    end
    omega(i+1,:)        = a.omega(1:SKIP:end);
end

% interpolate rho & c from the background model onto the grid used by the
% eigenfunctions. Then take the transpose resulting in a column vector.
% i.e. size(c) = Nt x 1
model = a.m; %This is the background model, the eigenfunctions/values computed from that
t     = a.t; 
k     = a.k(1:SKIP:end);
clear a
c     = interp1(model.t,model.c,t)';
rho   = interp1(model.t,model.rho,t)';
rhoc  = rho.*c;

% Compute the velocity eigenfunctions, u and v, as in Birch et al 2004 page 608
% They also represent the eigenfunctions xi_z and xi_h if w1 and w2 is used
u = zeros(size(efunc1));
v = zeros(size(efunc2));

for i = MIN_N:MAX_N   % for each mode
    for j = 1:Nk  % for each angular wavenumber
        
        % Vertical velocity, u, and pressure, P(t), as functions of y1 and 
        % y1 are computed according to Birch 2004 (Eqns A8-A9)
        % P(t) = -I    * sqrt[rho(t) c(t)] * y1(t)
        % u(t) = y2(t) / sqrt[rho(t) c(t)]
        if strcmp(p.eig_type, 'vel_y1y2')
            %P  = -li * squeeze(efunc1(i+1,j,:)) .* sqrt(rhoc);
            %tv = -li * (k(j)*1e-8) * P./rho/real(omega(i+1,j));
            tv = -(k(j)*1e-8) * squeeze(efunc1(i+1,j,:)) .* sqrt(rhoc) ./ rho / real(omega(i+1,j));
            tu = squeeze(efunc2(i+1,j,:)) ./ sqrt(rhoc);
        elseif strcmp(p.eig_type, 'vel_y1u')
            tv = -(k(j)*1e-8) * squeeze(efunc1(i+1,j,:)) .* sqrt(rhoc) ./ rho / real(omega(i+1,j));
            tu = squeeze(efunc2(i+1,j,:));
        elseif strcmp(p.eig_type, 'disp_w1w2')
            %P  = squeeze(efunc1(i+1,j,:)) .* sqrt(rhoc);
            %tv = (k(j)*1e-8) * P ./ rho / (real(omega(i+1,j)) * real(omega(i+1,j))); here tv = xi_h
            tv =  (k(j)*1e-8) * squeeze(efunc1(i+1,j,:)) .* sqrt(rhoc) ./ rho / (real(omega(i+1,j)) * real(omega(i+1,j))); 
            tu = squeeze(efunc2(i+1,j,:)) ./ sqrt(rhoc);
        elseif strcmp(p.eig_type, 'disp_w1xiz')
            tv =  (k(j)*1e-8) * squeeze(efunc1(i+1,j,:)) .* sqrt(rhoc) ./ rho / (real(omega(i+1,j)) * real(omega(i+1,j))); 
            tu = squeeze(efunc2(i+1,j,:));
        end
        
        % Normalize according to eqn 46, p 587, Birch 2004
        %n  = integrate(t,rhoc.*(abs(tu).^2+abs(tv).^2));    
        % Normalize according to Burstons new way. It includes a factor of
        % (2pi)^2 here rather than at the end of calc_green8c.m
        n  = (2*pi)^2*integrate(t,rhoc.*(abs(tu).^2+abs(tv).^2));  
        tu=tu/sqrt(n);
        tv=tv/sqrt(n);

        % Reshape
        v(i+1,j,:)=reshape(tv,1,1,Nt);
        u(i+1,j,:)=reshape(tu,1,1,Nt);
         
        % Try to ensure we dont get big jumps due to random sign switches
        if (j>1)
            In=1;
            t1=abs(u(i+1,j,In)+u(i+1,j-1,In))<abs(u(i+1,j,In)-u(i+1,j-1,In));
            t2=abs(v(i+1,j,In)+v(i+1,j-1,In))<abs(v(i+1,j,In)-v(i+1,j-1,In));
            if (t1 && t2)
                if(p.vol>=2)
                    disp('WARNING: load_eigs3.m : unexpected sign change');
                end
                v(i+1,j,:)=-v(i+1,j,:);
                u(i+1,j,:)=-u(i+1,j,:);
            end
        end
        
    end
end 
% save(['/scratch/seismo/burston/' p.eig_type '.mat'],'v','u','t','k');
% keyboard 
clear t1 t2 In tv tu y1 y2 P rho rhoc name SKIP
